declare type Expression = EInt | EVar | EFunc | ECall | EIf | ELet;
interface EInt {
    nodeType: "Int";
    value: number;
}
interface EVar {
    nodeType: "Var";
    name: string;
}
interface EFunc {
    nodeType: "Function";
    param: string;
    body: Expression;
}
interface ECall {
    nodeType: "Call";
    func: Expression;
    arg: Expression;
}
interface EIf {
    nodeType: "If";
    cond: Expression;
    trueBranch: Expression;
    falseBranch: Expression;
}
interface ELet {
    nodeType: "Let";
    name: string;
    rhs: Expression;
    body: Expression;
}
declare type Type = TNamed | TVar | TFun;
interface TNamed {
    nodeType: "Named";
    name: string;
}
interface TVar {
    nodeType: "Var";
    name: string;
}
interface TFun {
    nodeType: "Function";
    from: Type;
    to: Type;
}
declare type Context = {
    next: number;
    env: Env;
};
declare type Env = {
    [name: string]: Type;
};
declare type Substitution = {
    [key: string]: Type;
};
declare function typeToString(t: Type): string;
declare function infer(ctx: Context, e: Expression): [Type, Substitution];
export { typeToString, infer, Expression, Type, EFunc };
